/**
 * @file Click_Pollution_WSP2110.c
 * @brief <h2> Pollution click Example </h2>
 *
 * @par
 * Example for <a href="http://www.mikroe.com">MikroElektronika's</a>
 * Pollution click board.
 *
 *
 *
 ******************************************************************************/

/**
 * @page        Example
 * @date        15 Nov 2016
 * @author      Branislav Marton
 * @copyright   GNU Public License
 * @version     1.0.0 - Initial testing and verification
 */

/**
 * @page TEST_CFG Test Configurations
 *
 * ### Test configuration STM : ###
 * @par
 * -<b> MCU           </b> :    ATMEGA32
 * -<b> Dev. Board    </b> :    EasyAVR v7
 * -<b> Ext. Modules  </b> :    Pollution click board, LCD
 * -<b> SW            </b> :    mikroC PRO for AVR v6.1.2
 *
 */

/**
 * @page Dev. board setup
 *
 * 1) Put Pollution click in mikroBUS 1 slot.
 * 2) Button press level for PD set to GND.
 * 3) Set pull-up for PD7.
 */

/******************************************************************************
* Includes
*******************************************************************************/
#include <stdint.h>
#include <math.h>

#include "pollution.h"
#include "pollution_gui.h"

/******************************************************************************
* Preprocessors
*******************************************************************************/
#define DIR_OUT 1
#define DIR_IN  0

#define MAX_R_LOAD 9950.0

/******************************************************************************
* Typedefs
*******************************************************************************/


/******************************************************************************
* Variable Definitions
*******************************************************************************/
// LCD module connections
sbit LCD_RS at PORTA2_bit;
sbit LCD_EN at PORTD6_bit;
sbit LCD_D4 at PORTC4_bit;
sbit LCD_D5 at PORTC5_bit;
sbit LCD_D6 at PORTC6_bit;
sbit LCD_D7 at PORTC7_bit;

sbit LCD_RS_Direction at DDA2_bit;
sbit LCD_EN_Direction at DDD6_bit;
sbit LCD_D4_Direction at DDC4_bit;
sbit LCD_D5_Direction at DDC5_bit;
sbit LCD_D6_Direction at DDC6_bit;
sbit LCD_D7_Direction at DDC7_bit;
// End LCD module connections

sbit RST        at PORTA6_bit;
sbit RST_DIR    at DDA6_bit;

sbit TAST_PD7       at PIND7_bit;
sbit TAST_PD7_DIR   at DDD7_bit;

float v_load, v_load_old = 0;

float r_0;
float r_s;
float r_l, r_l_old = 0;

float ratio, ratio_old = 0;
float ppm, ppm_old = 0;

uint8_t button_state_rg0 = 0xFF;

uint8_t refresh_counter = 0;



/******************************************************************************
* Function Prototypes
*******************************************************************************/
static void system_init( void );
static void port_init( void );

static void calibration( void );
static void adjust_load();

uint8_t button_pressed(uint8_t state, uint8_t* old_state);

/******************************************************************************
* Private Functions
*******************************************************************************/

/***************************************************************************//**
 *  System initialisation.
 */
static void system_init( void )
{
    port_init();

    gui_init();  
}

/**
 *  MCU's port initialisation.
 */
static void port_init( void )
{
    RST_DIR = DIR_OUT;      // Configure RC1/RST as output.
    RST = 1;

    TAST_PD7_DIR = DIR_IN;  // Configure RG0 pin as input.
}

/***************************************************************************//**
 *  Procedure for sensor calibration.
 */
void calibration()
{
    gui_print_calibration_page();

    while( ! button_pressed(TAST_PD7, &button_state_rg0) )
    {
        // Refresh V load value on display every 250ms.
        if(! (refresh_counter++ % 10))
            v_load_old = gui_refresh_value(
                measure_v_load(), 
                v_load_old, 
                235, 80, 
                "%.3f"
            );

        Delay_ms(25);
    }
    // Save sensor resistance in normal condition with MAX R load.
    r_0 = sensor_resistance(measure_v_load(), MAX_R_LOAD);

    gui_clear_page();
}

/***************************************************************************//**
 *  Procedure for adjusting load.
 */
void adjust_load()
{
    gui_print_adjust_load_page();
    
    while( ! button_pressed(TAST_PD7, &button_state_rg0) )
    {
        // Refresh value on display every 250ms.
        if(!(refresh_counter++ % 10)) 
        {
            r_l = load_resistance(measure_v_load(), r_0);
            r_l_old = gui_refresh_value(r_l / 1000, r_l_old, 115, 80, "%.2fk");
        }
       
        Delay_ms(25);
    }
    // Save adjusted load resistance.
    r_l = load_resistance(measure_v_load(), r_0);

    gui_clear_page();
}

/***************************************************************************//**
 *  Function for detecting if button is pressed, with polling method.
 *  \param state Button state.
 *  \param old_state Latest detected state.
 *  \return Function returns 0xFF if falling edge is detected, 0x00 otherwise.
 */
uint8_t button_pressed(uint8_t state, uint8_t* old_state)
{
    if(!state && *old_state)
    {
        *old_state = state;
        return 0xFF;
    }

    *old_state = state;
    return 0x00;
}


/******************************************************************************
* Public Functions
*******************************************************************************/
void main()
{
    system_init();

    calibration();
    adjust_load();

    gui_print_measure_page();
    
    while(1)
    {
        r_s = sensor_resistance(measure_v_load(), r_l);
        
        ratio = r_s / r_0 > 1 ? 1.0 : r_s / r_0;

        ppm = ratio_to_ppm(ratio);
        ppm_old = gui_refresh_value(ppm, ppm_old, 235, 160, "%.1f");

        gui_update_inequality_sign(ppm);
        
        Delay_ms(250);
    }
}
/*************** END OF FUNCTIONS *********************************************/